@@ -1,12 +1,12 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE LambdaCase #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE TypeApplications #-}
+{-# LANGUAGE FlexibleInstances    #-}
+{-# LANGUAGE LambdaCase           #-}
+{-# LANGUAGE OverloadedStrings    #-}
+{-# LANGUAGE ScopedTypeVariables  #-}
+{-# LANGUAGE TypeApplications     #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 {-|
 Module      : HsLua.Module.DocLayout
-Copyright   : © 2020-2022 Albert Krewinkel
+Copyright   : © 2020-2024 Albert Krewinkel
 License     : MIT
 Maintainer  : Albert Krewinkel <albert+hslua@zeitkraut.de>
 
@@ -78,10 +78,6 @@ import Text.DocLayout (Doc, (<+>), ($$), ($+$))
 import qualified Data.Text as T
 import qualified Text.DocLayout as Doc
 
-#if ! MIN_VERSION_base(4, 11, 0)
-import Data.Monoid ((<>))
-#endif
-
 --
 -- Module
 --
@@ -98,6 +94,7 @@ documentedModule = Module
   , moduleDescription = description
   , moduleFunctions = functions
   , moduleOperations = []
+  , moduleTypeInitializers = [initType typeDoc]
   }
 
 --
@@ -105,7 +102,7 @@ documentedModule = Module
 --
 
 -- | Exposed fields.
-fields :: LuaError e => [Field e]
+fields :: forall e. LuaError e => [Field e]
 fields =
   [ blankline
   , cr
@@ -114,36 +111,40 @@ fields =
   ]
 
 -- | Wrapped and documented 'Doc.blankline' value.
-blankline :: LuaError e => Field e
+blankline :: forall e. LuaError e => Field e
 blankline = Field
   { fieldName = "blankline"
   , fieldDescription = "Inserts a blank line unless one exists already."
+  , fieldType = udTypeSpec @e typeDoc
   , fieldPushValue = pushDoc Doc.blankline
   }
 
 -- | Wrapped and documented 'Doc.cr' value.
-cr :: LuaError e => Field e
+cr :: forall e. LuaError e => Field e
 cr = Field
   { fieldName = "cr"
   , fieldDescription = "A carriage return. Does nothing if we're at " <>
                        "the beginning of a line; " <>
                        "otherwise inserts a newline."
+  , fieldType = udTypeSpec @e typeDoc
   , fieldPushValue = pushDoc Doc.cr
   }
 
 -- | Wrapped and documented 'Doc.empty' value.
-empty :: LuaError e => Field e
+empty :: forall e. LuaError e => Field e
 empty = Field
   { fieldName = "empty"
   , fieldDescription = "The empty document."
+  , fieldType = udTypeSpec @e typeDoc
   , fieldPushValue = pushDoc Doc.empty
   }
 
 -- | Wrapped and documented 'Doc.space' value.
-space :: LuaError e => Field e
+space :: forall e. LuaError e => Field e
 space = Field
   { fieldName = "space"
   , fieldDescription = "A breaking (reflowable) space."
+  , fieldType = udTypeSpec @e typeDoc
   , fieldPushValue = pushDoc Doc.space
   }
 
@@ -246,9 +247,11 @@ render = defun "render"
   <#> docParam "doc"
   <#> opt (integralParam "colwidth" "planned maximum line length")
   =#> functionResult pushText "Doc" "rendered doc"
-  #? ("Render a @'Doc'@. The text is reflowed on breakable spaces" <>
-      "to match the given line length. Text is not reflowed if the" <>
-      "line length parameter is omitted or nil.")
+  #? T.unlines
+     [ "Render a [[Doc]]. The text is reflowed on breakable spaces to"
+     , "match the given line length. Text is not reflowed if the line"
+     , "line length parameter is omitted or nil."
+     ]
 
 --
 -- Querying
@@ -268,7 +271,7 @@ offset = defun "offset"
   ### liftPure Doc.offset
   <#> docParam "doc"
   =#> integralResult "doc width"
-  #? "Returns the width of a `Doc` as number of characters."
+  #? "Returns the width of a [[Doc]] as number of characters."
 
 -- | Returns the minimal width of a @'Doc'@ when reflowed at
 -- breakable spaces.
@@ -277,7 +280,7 @@ min_offset = defun "min_offset"
   ### liftPure Doc.minOffset
   <#> docParam "doc"
   =#> integralResult "minimal possible width"
-  #? ("Returns the minimal width of a `Doc` when reflowed at " <>
+  #? ("Returns the minimal width of a [[Doc]] when reflowed at " <>
       "breakable spaces.")
 
 -- | Returns the column that would be occupied by the last laid
@@ -323,7 +326,7 @@ after_break = defun "after_break"
   ### liftPure Doc.afterBreak
   <#> textParam "text" "content to include when placed after a break"
   =#> docResult "new doc"
-  #? ("Creates a `Doc` which is conditionally included only if it " <>
+  #? ("Creates a [[Doc]] which is conditionally included only if it " <>
       "comes at the beginning of a line.\n\n" <>
       "An example where this is useful is for escaping line-initial " <>
       "`.` in roff man.")
@@ -426,7 +429,7 @@ inside = defun "inside"
   <#> docParam "start"
   <#> docParam "end"
   =#> docResult "enclosed contents"
-  #? "Encloses a `Doc` inside a start and end `Doc`."
+  #? "Encloses a [[Doc]] inside a start and end [[Doc]]."
 
 -- | Creates a block with the given width and content, aligned to
 -- the left.
